-- Author      : Kurapica
-- Create Date : 5/4/2012
-- ChangeLog

----------------------------------------------------------------------------------------------------------------------------------------
--- LayoutPanel
-- <br><br>inherit <a href="..\Base\Frame.html">Frame</a> For all methods, properties and scriptTypes
-- @name LayoutPanel
----------------------------------------------------------------------------------------------------------------------------------------

local version = 1
if not IGAS:NewAddon("IGAS.Widget.LayoutPanel", version) then
	return
end

class "LayoutPanel"
	inherit "Frame"

	local tinsert = tinsert
	local tremove = tremove
	local floor = floor
	local ceil = ceil
	local pcall = pcall

	_Use_ShortCut = true
	
	_Error_Widget = "%swidget not existed."
	_Error_Number = "%s%s must be a number, got %s."
	_Error_Percent = "%s%s must be 1 - 100."
	_Error_Zero = "%s%s can't be less than 0."
	_Error_Size = "%s%s can't be greater than parent's %s."
	_Error_Combine = "%sthe settings are oversize."

	local function ValidateSet(self, index, prefix, selfParam,
			startName, startUnitName, endName, endUnitName, sizeName, sizeUnitName,
			startValue, startUnit, endValue, endUnit, sizeValue, sizeUnit)
		local size = self[selfParam]
		local obj = GetWidget(self, index)
		local startFunc, endFunc, sizeFunc
		local minSize, remainPct, realSize

		prefix = prefix or ""
		remainPct = 100
		realSize = 0

		if obj then
			-- Check start
			if startName then
				if type(startValue) ~= "number" then
					error(_Error_Number:format(prefix, startName, type(startValue)), 3)
				end

				startUnit = Reflector.Validate(Unit, startUnit, startUnitName, prefix, 1)

				if startUnit == _PERCENT then
					if startValue < 0 or startValue > 100 then
						error(_Error_Percent:format(prefix, startName), 3)
					end
					remainPct = remainPct - startValue
					startFunc = function() return floor(self[selfParam] * startValue / 100) end
				elseif startUnit == _PIXEL then
					if startValue < 0 then
						error(_Error_Zero:format(prefix, startName), 3)
					elseif startValue > size then
						error(_Error_Size:format(prefix, startName, selfParam), 3)
					end
					startValue = floor(startValue)
					realSize = realSize + startValue
					startFunc = function() return startValue end
				end
			end

			-- Check end
			if endName then
				if type(endValue) ~= "number" then
					error(_Error_Number:format(prefix, endName, type(endValue)), 3)
				end

				endUnit = Reflector.Validate(Unit, endUnit, endUnitName, prefix, 1)

				if endUnit == _PERCENT then
					if endValue < 0 or endValue > 100 then
						error(_Error_Percent:format(prefix, endName), 3)
					end
					remainPct = remainPct - endValue
					endFunc = function() return floor(self[selfParam] * endValue / 100) end
				elseif endUnit == _PIXEL then
					if endValue < 0 then
						error(_Error_Zero:format(prefix, endName), 3)
					elseif endValue > size then
						error(_Error_Size:format(prefix, endName, selfParam), 3)
					end
					endValue = floor(endValue)
					realSize = realSize + endValue
					endFunc = function() return endValue end
				end
			end

			-- Check size
			if sizeName then
				if type(sizeValue) ~= "number" then
					error(_Error_Number:format(prefix, sizeName, type(sizeValue)), 3)
				end

				sizeUnit = Reflector.Validate(Unit, sizeUnit, sizeUnitName, prefix, 1)

				if sizeUnit == _PERCENT then
					if sizeValue < 0 or sizeValue > 100 then
						error(_Error_Percent:format(prefix, sizeName), 3)
					end
					remainPct = remainPct - sizeValue
					sizeFunc = function() return floor(self[selfParam] * sizeValue / 100) end
				elseif sizeUnit == _PIXEL then
					if sizeValue < 0 then
						error(_Error_Zero:format(prefix, sizeName), 3)
					elseif sizeValue > size then
						error(_Error_Size:format(prefix, sizeName, selfParam), 3)
					end
					sizeValue = floor(sizeValue)
					realSize = realSize + sizeValue
					sizeFunc = function() return sizeValue end
				end
			end

			if not startFunc then
				startFunc = function() return floor(self[selfParam] - endFunc() - sizeFunc()) end
			elseif not sizeFunc then
				sizeFunc = function() return floor(self[selfParam] - endFunc() - startFunc()) end
			end

			-- Final check
			if startFunc() < 0 or sizeFunc() < 0 or startFunc() + sizeFunc() > size then
				error(_Error_Combine:format(prefix), 3)
			end

			if realSize > 0 then
				minSize = ceil(realSize * 100 / remainPct)
			end

			self.__NeedUpdateMinSize = true

			if selfParam == "Width" then
				obj.__Layout_Left = startFunc
				obj.__Layout_Width = sizeFunc
				obj.__Layout_MinWidth = minSize
			elseif selfParam == "Height" then
				obj.__Layout_Top = startFunc
				obj.__Layout_Height = sizeFunc
				obj.__Layout_MinHeight = minSize
			end
		else
			error(_Error_Widget:format(prefix), 3)
		end
	end

	------------------------------------------------------
	-- Enum
	------------------------------------------------------
	_PERCENT = "PERCENT"
	_PIXEL = "PIXEL"

	enum "Unit" {
		PCT = _PERCENT,
		PX = _PIXEL,
	}

	------------------------------------------------------
	-- Script
	------------------------------------------------------


	------------------------------------------------------
	-- Method
	------------------------------------------------------

	------------------------------------
	--- Add Widget to the panel
	-- @name LayoutPanel:AddWidget
	-- @class function
	-- @param widget
	-- @return widget
	-- @return index
	------------------------------------
	function AddWidget(self, widget)
		self.__LayoutItems = self.__LayoutItems or {}

		if widget and Reflector.ObjectIsClass(widget, Region) then
			if self:GetChild(widget.Name) == widget then
				local idx

				for i = 1, #(self.__LayoutItems) do
					if self.__LayoutItems[i] == widget then
						idx = i
						break
					end
				end

				if idx then
					return self, idx
				end
			elseif self:GetChild(widget.Name) then
				error("Usage : LayoutPanel:AddWidget(widget) : a widget already existed with same name.", 2)
			elseif widget.Parent ~= self then
				widget.Parent = self
			end

			tinsert(self.__LayoutItems, widget)

			self:Layout()

			return self, #(self.__LayoutItems)
		else
			error("Usage : LayoutPanel:AddWidget(widget) : widget must be an object of System.Widget.Region.", 2)
		end
	end

	------------------------------------
	--- Insert Widget to the panel
	-- @name LayoutPanel:InsertWidget
	-- @class function
	-- @param index the index to be insert
	-- @param widget
	-- @return widget
	-- @return index
	------------------------------------
	function InsertWidget(self, widget, index)
		self.__LayoutItems = self.__LayoutItems or {}
		
		if index then
			local ok, obj
			
			ok, obj, index = pcall(GetWidget, self, index, true)
			
			if not (ok and obj and index) then
				error("Usage : LayoutPanel:InsertWidget(widget, index) : index not exist.", 2)
			end
		else
			index = #(self.__LayoutItems) + 1
		end

		if widget and Reflector.ObjectIsClass(widget, Region) then
			if self:GetChild(widget.Name) == widget then
				local idx

				for i = 1, #(self.__LayoutItems) do
					if self.__LayoutItems[i] == widget then
						idx = i
						break
					end
				end

				if idx then
					return self, idx
				end
			elseif self:GetChild(widget.Name) then
				error("Usage : LayoutPanel:InsertWidget(index, widget) : a widget already existed with same name.", 2)
			elseif widget.Parent ~= self then
				widget.Parent = self
			end

			tinsert(self.__LayoutItems, index, widget)

			self:Layout()

			return self, index
		else
			error("Usage : LayoutPanel:InsertWidget(index, widget) : widget must be an object of System.Widget.Region.", 2)
		end
	end

	------------------------------------
	--- Get Widget from the panel
	-- @name LayoutPanel:GetWidget
	-- @class function
	-- @param index|name index or the name that need to be removed
	-- @return widget
	-- @return index
	------------------------------------
	function GetWidget(self, index, needIndex)
		self.__LayoutItems = self.__LayoutItems or {}

		local obj

		if Reflector.ObjectIsClass(index, Region) then
			if self:GetChild(index.Name) == index then
				if not needIndex then
					return index
				end

				local idx

				for i = 1, #(self.__LayoutItems) do
					if self.__LayoutItems[i] == index then
						idx = i
						break
					end
				end

				if idx then
					return index, idx
				else
					error("Usage : LayoutPanel:GetWidget(widget[, needIndex]) : widget not added to the panel.", 2)
				end
			elseif self:GetChild(index.Name) then
				error("Usage : LayoutPanel:GetWidget(widget[, needIndex]) : a widget already existed with same name.", 2)
			end
		end

		if type(index) == "string" then
			obj = self:GetChild(index)

			if not needIndex or not obj then
				return obj
			end

			index = nil

			for i, v in ipairs(self.__LayoutItems) do
				if v == obj then
					index = i
					break
				end
			end
		end

		if type(index) == "number" then
			index = floor(index)

			if index > 0 and index <= #(self.__LayoutItems) then
				return self.__LayoutItems[index], index
			end
		end
	end

	------------------------------------
	--- Remove Widget to the panel
	-- @name LayoutPanel:RemoveWidget
	-- @class function
	-- @param index|name index or the name that need to be removed
	-- @param withoutDispose optional, true if need get the removed widget
	-- @return widget if withoutDispose is set to true
	------------------------------------
	function RemoveWidget(self, index, withoutDispose)
		local obj

		obj, index = GetWidget(self, index, true)

		if index and obj then
			tremove(self.__LayoutItems, index)

			if not withoutDispose then
				obj:Dispose()
			else
				return obj
			end
		end
	end

	------------------------------------
	--- Set Widget's Visible
	-- @name LayoutPanel:SetWidgetVisible
	-- @class function
	-- @param index|name index or the name
	-- @param visible
	------------------------------------
	function SetWidgetVisible(self, index, visible)
		local obj = GetWidget(self, index)

		visible = visible and true or false

		if obj then
			obj.__Layout_Visible = visible
			self:Layout()
		else
			error("Usage : LayoutPanel:SetWidgetVisible(index||name||widget, visible) : widget not existed.", 2)
		end

		return self
	end

	if _Use_ShortCut then SWV = SetWidgetVisible end
	
	------------------------------------
	--- Set Widget's left margin and right margin
	-- @name LayoutPanel:SetWidgetLeftRight
	-- @class function
	-- @param index|name index or the name
	-- @param left left margin value
	-- @param leftunit left margin's unit
	-- @param right right margin value
	-- @param rightunit right margin's unit
	------------------------------------
	function SetWidgetLeftRight(self, index, left, leftunit, right, rightunit)
		local prefix = "Usage : LayoutPanel:SetWidgetLeftRight(index||name||widget, left, leftunit, right, rightunit) : "

		ValidateSet(self, index, prefix, "Width",
			"left", "leftunit", "right", "rightunit", nil, nil,
			left, leftunit, right, rightunit, nil, nil)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWLR = SetWidgetLeftRight end
	
	------------------------------------
	--- Set Widget's left margin and width
	-- @name LayoutPanel:SetWidgetLeftWidth
	-- @class function
	-- @param index|name index or the name
	-- @param left left margin value
	-- @param leftunit left margin's unit
	-- @param width width value
	-- @param widthunit width unit
	------------------------------------
	function SetWidgetLeftWidth(self, index, left, leftunit, width, widthunit)
		local prefix = "Usage : LayoutPanel:SetWidgetLeftWidth(index||name||widget, left, leftunit, width, widthunit) :"
		
		ValidateSet(self, index, prefix, "Width",
			"left", "leftunit", nil, nil, "width", "widthunit",
			left, leftunit, nil, nil, width, widthunit)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWLW = SetWidgetLeftWidth end
	
	------------------------------------
	--- Set Widget's right margin and width
	-- @name LayoutPanel:SetWidgetRightWidth
	-- @class function
	-- @param index|name index or the name
	-- @param right right margin value
	-- @param rightunit right margin's unit
	-- @param width width value
	-- @param widthunit width unit
	------------------------------------
	function SetWidgetRightWidth(self, index, right, rightunit, width, widthunit)
		local prefix = "Usage : LayoutPanel:SetWidgetRightWidth(index||name||widget, right, rightunit, width, widthunit) : "

		ValidateSet(self, index, prefix, "Width",
			nil, nil, "right", "rightunit", "width", "widthunit",
			nil, nil, right, rightunit, width, widthunit)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWRW = SetWidgetRightWidth end
	
	------------------------------------
	--- Set Widget's horizontal position
	-- @name LayoutPanel:SetWidgetRightWidth
	-- @class function
	-- @param index|name index or the name
	-- @param align begin|stretch|end
	------------------------------------
	-- function SetWidgetHorizontalPosition(self, index, align)end

	------------------------------------
	--- Set Widget's top margin and bottom margin
	-- @name LayoutPanel:SetWidgetTopBottom
	-- @class function
	-- @param index|name index or the name
	-- @param top top margin value
	-- @param topunit top margin's unit
	-- @param bottom bottom margin value
	-- @param bottomunit bottom margin's unit
	------------------------------------
	function SetWidgetTopBottom(self, index, top, topunit, bottom, bottomunit)
		local prefix = "Usage : LayoutPanel:SetWidgetTopBottom(index||name||widget, top, topunit, bottom, bottomunit) : "

		ValidateSet(self, index, prefix, "Height",
			"top", "topunit", "bottom", "bottomunit", nil, nil,
			top, topunit, bottom, bottomunit, nil, nil)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWTB = SetWidgetTopBottom end
	
	------------------------------------
	--- Set Widget's top margin and height
	-- @name LayoutPanel:SetWidgetTopHeight
	-- @class function
	-- @param index|name index or the name
	-- @param top top margin value
	-- @param topunit top margin's unit
	-- @param height height value
	-- @param heightunit height's unit
	------------------------------------
	function SetWidgetTopHeight(self, index, top, topunit, height, heightunit)
		local prefix = "Usage : LayoutPanel:SetWidgetTopHeight(index||name||widget, top, topunit, height, heightunit) : "

		ValidateSet(self, index, prefix, "Height",
			"top", "topunit", nil, nil, "height", "heightunit",
			top, topunit, nil, nil, height, heightunit)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWTH = SetWidgetTopHeight end
	
	------------------------------------
	--- Set Widget's top margin and height
	-- @name LayoutPanel:SetWidgetBottomHeight
	-- @class function
	-- @param index|name index or the name
	-- @param bottom top margin value
	-- @param bottomunit bottom margin's unit
	-- @param height height value
	-- @param heightunit height's unit
	------------------------------------
	function SetWidgetBottomHeight(self, index, bottom, bottomunit, height, heightunit)
		local prefix = "Usage : LayoutPanel:SetWidgetBottomHeight(index||name||widget, bottom, bottomunit, height, heightunit) : "

		ValidateSet(self, index, prefix, "Height",
			nil, nil, "bottom", "bottomunit", "height", "heightunit",
			nil, nil, bottom, bottomunit, height, heightunit)

		self:Layout()
		
		return self
	end

	if _Use_ShortCut then SWBH = SetWidgetBottomHeight end
	
	------------------------------------
	--- Set Widget's horizontal position
	-- @name LayoutPanel:SetWidgetRightWidth
	-- @class function
	-- @param index|name index or the name
	-- @param align begin|stretch|end
	------------------------------------
	-- function SetWidgetVerticalPosition(self, index, align)end

	------------------------------------
	--- Update layout
	-- @name LayoutPanel:Layout
	-- @class function
	------------------------------------
	function Layout(self)
		if self.__SuspendLayout or not self.__LayoutItems then
			return
		end

		local obj, left, top, width, height
		local minWidth, minHeight = 0, 0
		
		for i = 1, #(self.__LayoutItems) do
			obj = self.__LayoutItems[i]
			
			-- Check MinSize
			if obj.__Layout_MinWidth and obj.__Layout_MinWidth > minWidth then
				minWidth = obj.__Layout_MinWidth
			end
			
			if obj.__Layout_MinHeight and obj.__Layout_MinHeight > minHeight then
				minHeight = obj.__Layout_MinHeight
			end
			
			-- Calculate obj's new position
			left = obj.__Layout_Left and obj.__Layout_Left() or 0
			width = obj.__Layout_Width and obj.__Layout_Width() or self.Width
			top = obj.__Layout_Top and obj.__Layout_Top() or 0
			height = obj.__Layout_Height and obj.__Layout_Height() or self.Height
			
			if left and width and top and height then
				obj:ClearAllPoints()
				
				obj:SetPoint("LEFT", self, "LEFT", left, 0)
				obj:SetPoint("TOP", self, "TOP", -top, 0)
				
				obj.Width = width
				obj.Height = height
			end
			
			if obj.__Layout_Visible ~= nil then
				obj.Visible = obj.__Layout_Visible
			end
		end
		
		if self.__NeedUpdateMinSize then
			self.__NeedUpdateMinSize = nil
			self:SetMinResize(minWidth, minHeight)
		end
	end

	------------------------------------
	--- stop the refresh of the list
	-- @name List:SuspendLayout
	-- @class function
	-- @usage List:SuspendLayout()
	------------------------------------
	function SuspendLayout(self)
		self.__SuspendLayout = true
	end

	------------------------------------
	--- resume the refresh of the list
	-- @name List:ResumeLayout
	-- @class function
	-- @usage List:ResumeLayout()
	------------------------------------
	function ResumeLayout(self)
		self.__SuspendLayout = nil
		self:Layout()
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------


	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnSizeChanged(self)
		self:Layout()
	end

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function LayoutPanel(name, parent)
		local panel = Frame(name, parent)

		panel.OnSizeChanged = panel.OnSizeChanged + OnSizeChanged

		return panel
	end
endclass "LayoutPanel"